@@ -101,6 +101,21 @@ pub fn is_positive_definite(m: &[f64]) -> bool {
     true
 }
 
+/// Calculates the parity of a swap permutation array ipiv (e.g. that you get as an output from
+/// Lapack).
+pub fn ipiv_parity(ipiv: &[i32]) -> i32 {
+    let mut perm = ipiv.to_owned();
+    let mut par = 0;
+    for i in 0..perm.len() {
+        if perm[i] != i as i32 {
+            let j = perm[i] as usize;
+            perm.swap(i, j);
+            par += 1;
+        }
+    }
+    (-1_i32).pow(par)
+}
+
 pub fn diag(a: &[f64]) -> Vector {
     let n = is_square(a).unwrap();
     let mut results = Vector::new(Vec::with_capacity(n));
@@ -136,7 +151,7 @@ pub fn col_to_row_major(a: &[f64], nrows: usize) -> Vec<f64> {
 
 /// Transpose a matrix.
 pub fn transpose(a: &[f64], nrows: usize) -> Vec<f64> {
-    let ncols = is_matrix(&a, nrows).unwrap();
+    let ncols = is_matrix(a, nrows).unwrap();
 
     let mut at = Vec::with_capacity(a.len());
 
@@ -161,7 +176,7 @@ pub fn diag_matrix(a: &[f64]) -> Vector {
 
 /// Given an n by n matrix, invert it. The resulting matrix is returned as a flattened array.
 pub fn invert_matrix(matrix: &[f64]) -> Vec<f64> {
-    let n = is_square(&matrix).unwrap();
+    let n = is_square(matrix).unwrap();
     #[cfg(feature = "lapack")]
     {
         let n = n as i32;
@@ -228,7 +243,7 @@ pub fn solve_sys(a: &[f64], b: &[f64]) -> Vec<f64> {
     #[cfg(not(feature = "lapack"))]
     {
         let mut solutions = Vec::with_capacity(b.len());
-        let b = row_to_col_major(&b, n);
+        let b = row_to_col_major(b, n);
 
         if is_positive_definite(a) {
             let l = cholesky(a);
@@ -279,11 +294,11 @@ pub fn solve(a: &[f64], b: &[f64]) -> Vec<f64> {
 
     #[cfg(not(feature = "lapack"))]
     {
-        if is_positive_definite(&a) {
-            let l = cholesky(&a);
+        if is_positive_definite(a) {
+            let l = cholesky(a);
             cholesky_solve(&l, b)
         } else {
-            let (lu, piv) = lu(&a);
+            let (lu, piv) = lu(a);
             lu_solve(&lu, &piv, b)
         }
     }
@@ -310,12 +325,12 @@ pub fn matmul_blocked(
     let mut c = vec![0.; m * n];
 
     let a = if transpose_a {
-        transpose(&a, rows_a)
+        transpose(a, rows_a)
     } else {
         a.to_vec()
     };
     let b = if transpose_b {
-        transpose(&b, rows_b)
+        transpose(b, rows_b)
     } else {
         b.to_vec()
     };
@@ -405,6 +420,11 @@ pub fn matmul(
 
     #[cfg(not(feature = "blas"))]
     {
+        // if transposing both, use the identity A^T.B^T = (A.B)^T
+        if transpose_a && transpose_b {
+            return transpose(&matmul(a, b, rows_a, rows_b, false, false), cols_a);
+        }
+
         let m = if transpose_a { cols_a } else { rows_a };
         let l = if transpose_a { rows_a } else { cols_a };
         let n = if transpose_b { rows_b } else { cols_b };
@@ -412,12 +432,12 @@ pub fn matmul(
         let mut c = vec![0.; m * n];
 
         let a = if transpose_a {
-            transpose(&a, rows_a)
+            transpose(a, rows_a)
         } else {
             a.to_vec()
         };
         let b = if transpose_b {
-            transpose(&b, rows_b)
+            transpose(b, rows_b)
         } else {
             b.to_vec()
         };
@@ -514,6 +534,29 @@ pub fn sum(x: &[f64]) -> f64 {
     }
 }
 
+/// Calculates the logarithm of the sum of the exponentials in a stable manner.
+/// Applies the [LogSumExp](https://en.wikipedia.org/wiki/LogSumExp) operator to the array.
+pub fn logsumexp(x: &[f64]) -> f64 {
+    let xmax = max(x);
+    x.iter().map(|v| (v - xmax).exp()).sum::<f64>().ln() + xmax
+    // let xmax = x.max();
+    // (x - xmax).exp().sum().ln() + xmax
+}
+
+/// Calculates the logarithm of the mean of the exponentials in a stable manner.
+/// Applies the LogMeanExp operator (like the LogSumExp, but mean instead of sum) to the array.
+pub fn logmeanexp(x: &[f64]) -> f64 {
+    let xmax = max(x);
+    (x.iter().map(|v| (v - xmax).exp()).sum::<f64>() / x.len() as f64).ln() + xmax
+    // let xmax = x.max();
+    // (x - xmax).exp().mean().ln() + xmax
+}
+
+/// Calculates the product of a vector.
+pub fn prod(x: &[f64]) -> f64 {
+    x.iter().product()
+}
+
 /// Calculates the dot product of two equal-length vectors. When the feature "blas" is enabled,
 /// uses `ddot` from BLAS. Otherwise, uses a length-8 unrolled loop.
 pub fn dot(x: &[f64], y: &[f64]) -> f64 {
@@ -534,6 +577,7 @@ pub fn dot(x: &[f64], y: &[f64]) -> f64 {
         for i in 0..chunks {
             let idx = i * 8;
             assert!(n > idx + 7);
+
             s += x[idx] * y[idx]
                 + x[idx + 1] * y[idx + 1]
                 + x[idx + 2] * y[idx + 2]
@@ -555,7 +599,7 @@ pub fn dot(x: &[f64], y: &[f64]) -> f64 {
 
 /// Calculates the norm of a vector.
 pub fn norm(x: &[f64]) -> f64 {
-    dot(&x, &x).sqrt()
+    dot(x, x).sqrt()
 }
 
 /// Calculates the infinity norm of a matrix. That is, it sums the absolute values along each row,
